# pilot_script.sh

This is the repository for the pilot script used by REPAST-PALMS FutureGateway application.
The script accepts different input parameters accordingly to the action to be performed as depicted below:

* For firs time or any further execution of REPAST PALMS:
```
./pilot_script.sh <submit> <user_name> <parameter_http_url> <model_http_url> [port ftp_password]
```

* To list files associated to the allocated resource
```
./pilot_script.sh <list> <user_name> <port>
```

* To release allocated resources
```
./pilot_script.sh <release> <user_name>
```

## First time execution
Below an example of 1st time `submit` execution:

```bash
$ ./pilot_script.sh submit testusr http://jobserver2.hopto.org/repast/PALMS/model.tar http://jobserver2.hopto.org/repast/PALMS/input/batch_params.xml_0
WARNING: The Docker Engine you're using is running in swarm mode.

Compose does not use swarm mode to deploy services to multiple nodes in a swarm. All containers will be scheduled on the current node.

To deploy your application across the swarm, use `docker stack deploy`.

Creating network "futuregateway_palms_network_testusr" with the default driver
Creating volume "futuregateway_palms_ftpdir_testusr" with default driver
Creating futuregateway_ftpd_testusr_1 ... 
Creating futuregateway_ftpd_testusr_1 ... done
**************** checkpoint 00 *****************
testusr BE896fGl ftp://ftpd_testusr/ http://jobserver2.hopto.org/repast/PALMS/model.tar http://jobserver2.hopto.org/repast/PALMS/input/batch_params.xml_0
**************** checkpoint 01 *****************
**************** checkpoint 1 *****************
  % Total    % Received % Xferd  Average Speed   Time    Time     Time  Current
                                 Dload  Upload   Total   Spent    Left  Speed
100 3544k  100 3544k    0     0  9469k      0 --:--:-- --:--:-- --:--:-- 9450k
**************** checkpoint 2 *****************
**************** checkpoint 3 *****************
  % Total    % Received % Xferd  Average Speed   Time    Time     Time  Current
                                 Dload  Upload   Total   Spent    Left  Speed
100  2498  100  2498    0     0  38717      0 --:--:-- --:--:-- --:--:-- 39031
pa_model
/opt/repast/RepastTest2/MyModels/pa_model
log4j:WARN No appenders could be found for logger (org.java.plugin.boot.DefaultPluginsCollector).
log4j:WARN Please initialize the log4j system properly.
log4j:WARN See http://logging.apache.org/log4j/1.2/faq.html#noconfig for more info.
RepastInit.run()
BatchRunner.run()
  % Total    % Received % Xferd  Average Speed   Time    Time     Time  Current
                                 Dload  Upload   Total   Spent    Left  Speed
100 40960    0     0  100 40960      0  1476k --:--:-- --:--:-- --:--:-- 1481k
  % Total    % Received % Xferd  Average Speed   Time    Time     Time  Current
                                 Dload  Upload   Total   Spent    Left  Speed
100 40960  100 40960    0     0  19.5M      0 --:--:-- --:--:-- --:--:-- 19.5M
{
 "user": "testusr",
 "model": "http://jobserver2.hopto.org/repast/PALMS/model.tar",
 "params": "http://jobserver2.hopto.org/repast/PALMS/input/batch_params.xml_0",
 "container": {
   "port": "40000",
   "name": "testusr_40000_oarpalms",
   "id": "ecbeabf11796"},
 "output": {
   "ftp_user": "testusr",
   "ftp_pass": "BE896fGl",
   "file_name": "out_batch_params.xml_0.tar",
   "url": "http://fgsg.ct.infn.it:40000/testusr/out_batch_params.xml_0.tar"}
}
```

The script returns **always** zero as script return code, a json stream in its **standard output** and the file `output.tar` containing the output of PALMS execution. The json file contains several information about the `submit` action and the most important values are:

* The **port** number of an HTTP server, generated automatically by the script.
* The **ftp_pass** string, representing the ftp password used to store the PALMS output.

The `pilot_script.sh` executes a docker-compose file made by two nodes, one for the PALMS execution and the second to store produced output files generated by the computation. Such file is uploaded into the ftp server and in the same time it will be available for download through HTTTP, since the FTP content is published by and HTTPD daemon running in the second node. The first node terminates after the PALMS execution while the second node will remain up and running in order to store or retrieve files generated by PALMS executions.
The port number refers to the port number of the listening HTTPD server containing the user' outputs.

## Further executions

Below an example of further execution, where the produced output will be added to the previous executions:

```bash
$ ./pilot_script.sh submit testusr http://jobserver2.hopto.org/repast/PALMS/model.tar http://jobserver2.hopto.org/repast/PALMS/input/batch_params.xml_2 40000 BE896fGl
WARNING: The Docker Engine you're using is running in swarm mode.

Compose does not use swarm mode to deploy services to multiple nodes in a swarm. All containers will be scheduled on the current node.

To deploy your application across the swarm, use `docker stack deploy`.

Starting futuregateway_ftpd_testusr_1 ... done
**************** checkpoint 00 *****************
testusr BE896fGl ftp://ftpd_testusr/ http://jobserver2.hopto.org/repast/PALMS/model.tar http://jobserver2.hopto.org/repast/PALMS/input/batch_params.xml_2
**************** checkpoint 01 *****************
**************** checkpoint 1 *****************
  % Total    % Received % Xferd  Average Speed   Time    Time     Time  Current
                                 Dload  Upload   Total   Spent    Left  Speed
100 3544k  100 3544k    0     0  11.9M      0 --:--:-- --:--:-- --:--:-- 11.9M
**************** checkpoint 2 *****************
**************** checkpoint 3 *****************
  % Total    % Received % Xferd  Average Speed   Time    Time     Time  Current
                                 Dload  Upload   Total   Spent    Left  Speed
100  2498  100  2498    0     0  38429      0 --:--:-- --:--:-- --:--:-- 39031
pa_model
/opt/repast/RepastTest2/MyModels/pa_model
log4j:WARN No appenders could be found for logger (org.java.plugin.boot.DefaultPluginsCollector).
log4j:WARN Please initialize the log4j system properly.
log4j:WARN See http://logging.apache.org/log4j/1.2/faq.html#noconfig for more info.
RepastInit.run()
BatchRunner.run()
  % Total    % Received % Xferd  Average Speed   Time    Time     Time  Current
                                 Dload  Upload   Total   Spent    Left  Speed
100 40960    0     0  100 40960      0  1486k --:--:-- --:--:-- --:--:-- 1538k
  % Total    % Received % Xferd  Average Speed   Time    Time     Time  Current
                                 Dload  Upload   Total   Spent    Left  Speed
100 40960  100 40960    0     0  8000k      0 --:--:-- --:--:-- --:--:-- 8000k
{
 "user": "testusr",
 "model": "http://jobserver2.hopto.org/repast/PALMS/model.tar",
 "params": "http://jobserver2.hopto.org/repast/PALMS/input/batch_params.xml_2",
 "container": {
   "port": "40000",
   "name": "testusr_40000_oarpalms",
   "id": "ecbeabf11796"},
 "output": {
   "ftp_user": "testusr",
   "ftp_pass": "BE896fGl",
   "file_name": "out_batch_params.xml_2.tar",
   "url": "http://fgsg.ct.infn.it:40000/testusr/out_batch_params.xml_2.tar"}
}
```

This time the `pilot_script` has two more arguments, the first contains the port number generated during the 1st execution and the 2nd the password of the FTP server still generated during the 1st execution.

## Output retrieval from HTTPD

To retrieve the list of the user output files:

```bash
./pilot_script.sh list testusr 40000 | jq .
{
  "files": [
    {
      "file": "out_batch_params.xml_0.tar",
      "url": "http://fgsg.ct.infn.it:40000/testusr/out_batch_params.xml_0.tar"
    },
    {
      "file": "out_batch_params.xml_2.tar",
      "url": "http://fgsg.ct.infn.it:40000/testusr/out_batch_params.xml_2.tar"
    }
  ]
}
```

To retrieve one of the files, just execute:

```bash
curl -s http://fgsg.ct.infn.it:40000/testusr/out_batch_params.xml_2.tar > out_batch_params.xml_2.tar
```

## Release resources

To release allocated resources to the user, execute:

```bash
./pilot_script.sh release testusr
```

Once the release action completes it will be not possible to retrieve output files anymore.

